package com.jclark.xsl;

class DescendantsQuery implements SubQuery {
  private Matchable pattern;
  
  DescendantsQuery(Matchable pattern) {
    this.pattern = pattern;
  }

  public SubQuery createUnion(SubQuery query) {
    if (query instanceof DescendantsQuery) {
      DescendantsQuery dq = (DescendantsQuery)query;
      if (pattern instanceof Matchable
	  && dq.pattern instanceof Matchable)
	return new DescendantsQuery(new OrPattern((Matchable)pattern,
	                                          (Matchable)dq.pattern));
    }
    return null;
  }

  private static class Iterator implements NodeIterator {
    private int depth;
    private NodeIterator iter;
    Node parent;
    private Matchable pattern;
    private ProcessContext context;

    Iterator(Matchable pattern, ProcessContext context, Node node) {
      this.pattern = pattern;
      this.context = context;
      depth = 0;
      iter = node.getChildren(true);
      parent = node;
    }

    public Node next() {
      for (;;) {
	Node node = iter.next();
	if (node != null) {
	  parent = node;
	  iter = node.getChildren(true);
	  if (pattern.matches(context, node, depth++))
	    return node;
	}
	else {
	  if (depth == 0)
	    break;
	  depth--;
	  iter = parent.getFollowingSiblings(true);
	  parent = parent.getParent();
	}
      }
      return null;
    }
  }

  public NodeIterator select(ProcessContext context, Node node) {
    return new Iterator(pattern, context, node);
  }
}
