package com.jclark.xsl;

import java.util.Hashtable;

class MacroProcessContext implements ProcessContext {
  private ProcessContext context;
  private Node invokeNode;
  private Hashtable arguments = new Hashtable();
  private Action contents;

  MacroProcessContext(ProcessContext context,
		      Node invokeNode,
		      AttributeList formals,
		      AttributeList actuals,
		      Action contents) throws XSLException {
    this.context = context;
    this.invokeNode = invokeNode;
    this.contents = contents;
    for (int i = 0; i < formals.getAttributeCount(); i++) {
      Name name = formals.getAttributeName(i);
      String actual = actuals.getAttributeValue(name);
      if (actual == null)
	arguments.put(name, formals.getAttributeValue(i));
      else
	arguments.put(name, actual);
    }
  }

  public
  void process(Node node, Result result) throws XSLException {
    context.process(node, result);
  }
  
  public
  boolean getStripSource(Node parent) {
    return context.getStripSource(parent);
  }    
  
  public
  boolean hasId(Node node, String name) {
    return context.hasId(node, name);
  }

  public
  Macro getMacro(Name name) {
    return context.getMacro(name);
  }

  public
  AttributeListTemplate getNamedAttributeSet(Name name) {
    return context.getNamedAttributeSet(name);
  }

  public
  String getMacroArgument(Name name) {
    return (String)arguments.get(name);
  }

  public
  String getConstant(Name name) throws XSLException {
    return context.getConstant(name);
  }

  public
  Node getElementWithId(String name) {
    return context.getElementWithId(name);
  }

  public
  void processContents(Result result) throws XSLException {
    contents.invoke(context, invokeNode, result);
  }
}
