package com.jclark.xsl;

import java.util.Vector;
import java.util.Enumeration;
import java.util.Hashtable;

class RuleSet {
  protected static abstract
  class Rule {
    final Matcher matcher;
    int priority;
    Rule(Matcher matcher, int priority) {
      this.priority = priority;
      this.matcher = matcher;
    }
    static
    boolean isBetter(Rule rule1, Rule rule2) {
      int s1 = rule1.matcher.getSpecificity();
      int s2 = rule2.matcher.getSpecificity();
      if (s1 == s2)
	return rule1.priority > rule2.priority;
      return s1 > s2;
    }
  }

  private Hashtable nameRules = new Hashtable();
  private Vector typeRules[] = new Vector[Node.N_TYPES];
  private Vector anyNameRules = new Vector();

  RuleSet() {
    for (int i = 0; i < Node.N_TYPES; i++)
      typeRules[i] = new Vector();
  }

  protected Enumeration potentialRules(Node node) {
    Name nodeName = node.getName();
    if (nodeName != null) {
      Vector rules = (Vector)nameRules.get(nodeName);
      if (rules != null)
	return rules.elements();
    }
    return typeRules[node.getType()].elements();
  }

  protected void add(Rule rule) {
    Name matchName = rule.matcher.getMatchName();
    if (matchName == null) {
      byte matchNodeType = rule.matcher.getMatchNodeType();
      typeRules[matchNodeType].addElement(rule);
      switch (matchNodeType) {
      case Node.ELEMENT:
      case Node.ATTRIBUTE:
	for (Enumeration enum = nameRules.elements(); enum.hasMoreElements();)
	  ((Vector)enum.nextElement()).addElement(rule);
	anyNameRules.addElement(rule);
	break;
      }
    }
    else {
      Vector v = (Vector)nameRules.get(matchName);
      if (v == null) {
	v = (Vector)anyNameRules.clone();
	nameRules.put(matchName, v);
      }
      v.addElement(rule);
    }
  }

  void sort() {
    for (Enumeration enum = nameRules.elements(); enum.hasMoreElements();)
      sortRulesVector((Vector)enum.nextElement());
    for (int i = 0; i < Node.N_TYPES; i++)
      sortRulesVector(typeRules[i]);
  }
  
  private static
  void sortRulesVector(Vector v) {
    // Insertion sort
    for (int i = 1; i < v.size(); i++) {
      Rule rule = (Rule)v.elementAt(i);
      int j;
      for (j = i; j > 0; j--) {
	Rule tem = (Rule)v.elementAt(j - 1);
	if (Rule.isBetter(tem, rule))
	  break;
	v.setElementAt(tem, j);
      }
      v.setElementAt(rule, j);
    }
  }
}
